
export interface IVeetingBlocksOptions {
	whitelabelDomain: string;
	version?: string;
	development?: boolean;
	initialized?: () => void;
}

export enum MeetingType {
	standard = "standard",
	offTheRecord = "offTheRecord",
	boardroom = "boardroom",
	classroom = "classroom",
	audiobridge = "audiobridge"
}

export enum MeetingRecordingType {
	full = "full",
	partial = "partial"
}

export enum MeetingRoomAuthType {
	none = "none",
	invited = "invited",
	accountMember = "accountMember",
	platformMember = "platformMember",
	external = "external",
	lobby = "lobby"
}

export enum MeetingRoomLayout {
	classic = "classic",
	presentation = "presentation",
	template = "template"
}

export enum Availability {
	required = "required",
	optional = "optional",
	hidden = "hidden"
}

export interface IMeetingRoomConfig {
	isNamedRoom: boolean;
	isBreakoutRoom: boolean;
	meetingId: string;
	roomId: string;
	id: string;
	topic: string;
	meetingType: MeetingType;
	dialInEnabled: boolean;
	soleilEnabled: boolean;
	languageChannelsEnabled: boolean;
	languageChannels?: string[];
	passwordProtected: boolean;
	hasVideo: boolean;
	startTime: number;
	endTime: number;
	isOpen: boolean;
	isPreOpen: boolean;
	isClosed: boolean;
	isActive: boolean;
	isDemoMeeting: boolean;
	isPromoMeeting: boolean;
	isFreeMeeting: boolean;
	isRecorded: boolean;
	recordingType?: MeetingRecordingType;
	isAccountValid: boolean;
	maxNumberOfParticipants?: number;
	maxNumberOfPublishers?: number;
	organizerId: string;
	accountId: string;
	logoFileName?: string;
	disableLogs?: boolean;
	broadcastingEnabled?: boolean;
	participantsEmail?: Availability;
	authType?: MeetingRoomAuthType;
	meetingRoomLayout?: MeetingRoomLayout;
	joinMeetingSound?: string;
	leaveMeetingSound?: string;
	closingMeetingSound?: string;
	closedMeetingSound?: string;
	mediaPlayerBroadcastVolumeChange?: boolean;
	mediaPlayerMuteVolumeBeforeOpeningTab?: boolean;
}

export enum MeetingRoomVideoResolution {
	"1280x960" = "1280x960",
	"1280x720" = "1280x720",
	"960x720" = "960x720",
	"960x540" = "960x540",
	"640x480" = "640x480",
	"640x360" = "640x360",
	"320x240" = "320x240",
	"320x180" = "320x180",
	"160x120" = "160x120"
}

export interface ICustomMessage {
	from?: string;
	to?: string;
	data?: any;
	onlyToModerators: boolean;
}

export type SecondaryVideoPosition = "none" | "left" | "top" | "right" | "bottom";
export enum SecondaryVideoSize {
	small = 0.15,
	large = 0.5
}

export interface IVideoPosition {
	participantId: string;
	width: number;
	height: number;
	top: number;
	left: number;
	zIndex: number;
	cssClasses?: string[];
}

export interface IVideoDisplayConfig {
	videoPositions: IVideoPosition[];
	hiddenContainers: boolean[];
}

export interface IMediaInformation {
	audioMuted?: boolean;
	videoMuted?: boolean;
	hasAudio?: boolean;
	hadAudio?: boolean;
	hasVideo?: boolean;
	// Bad network connectivity sometimes stops the video
	// In such a case hasVideo is false and hadVideo is true --> we expect video to come back
	hadVideo?: boolean;
	videoWidth?: number;
	videoHeight?: number;
}

export interface IParticipantsMediaInformation {
	[participantId: string]: IMediaInformation;
}

export interface IVideoDisplayCalculator {
	calculatePositions(
		containerWidth: number,
		containerHeight: number,
		hasSecondaryArea: boolean,
		secondaryPosition: SecondaryVideoPosition,
		secondarySize: SecondaryVideoSize,
		primaryParticipants: string[],
		secondaryParticipants: string[],
		participantsOrder: string[],
		particpantsMediaInformation: IParticipantsMediaInformation,
		hasSelfView: boolean,
		isSelfviewInSecondary: boolean,
		isTVMode: boolean
	): IVideoDisplayConfig;
}

export interface IApiParticipant {
	id: string;
	name: string;
	muted?: boolean;
	handRaised?: boolean;
	fromPSTN?: boolean;
	hasVideo?: boolean;
	hadVideo?: boolean;
	joinedAt: number;
}

export interface IApiChatMessage {
	fromParticipantId: string;
	fromParticipantName: string;
	message: string;
}

export interface IApiRemainingTimeUpdate {
	remainingSeconds: number;
}

export interface IScreenshareState {
	active: boolean;
}

export interface IMediaDeviceSettings {
	videoResolution: string;
	audioInputDeviceId: string;
	audioOutputDeviceId: string;
	videoInputDeviceId: string;
}

export enum MeetingRoomApiEvent {
	beforeConnecting = "beforeConnecting",
	serverConnected = "serverConnected", // WebRTC connection established
	connected = "connected", // WebRTC connection established
	disconnected = "disconnected", // WebRTC connection disconnected
	leave = "leave",
	participantsUpdated = "participantsUpdated",
	chatMessage = "chatMessage",
	privateChatMessage = "privateChatMessage",
	meetingDurationUpdated = "meetingDurationUpdated",
	meetingRoomConfigUpdated = "meetingRoomConfigUpdated",
	customMessage = "customMessage",
	screenshareStateChange = "screenshareStateChange"
}

export type ApiEventCallback = (payload: void | IApiParticipant[] | IApiChatMessage | IApiRemainingTimeUpdate | IMeetingRoomConfig | ICustomMessage | IScreenshareState) => void;

export interface IMeetingRoomApi {
	// Returns the current Veeting version string, i.e. 6.8.0
	getVersion(): string;
	// Returns the current participants list
	getParticipantsList(): IApiParticipant[];
	// Enables follow me (requires moderator rights)
	enableFollowMe(enabled: boolean): void;
	// Mutes video
	muteVideo(muted: boolean): void;
	// Toggles video mute
	toggleMuteVideo(): void;
	// Mutes audio
	muteAudio(muted: boolean): void;
	// Toggles audio mute
	toggleMuteAudio(): void;
	// Sets the media stream constraints for getUserMedia calls. If merge is true then mediaStreamConstraints will be merged with the internal constraints
	setMediaStreamConstraints(mediaStreamConstraints: MediaStreamConstraints, merge?: boolean /* default = true*/): void;
	// Will be ignored if the user is currently connected.
	// Users will automatically be connected when they join the meeting
	connect(audioOnly: boolean /* default false*/): void;
	// Will be ignored if the user is currently disconnected
	// "connected" refers to the WebRTC streams only. Users can participate on
	// the whiteboard even though they are disconnected
	disconnect(): void;
	// Sets the video input device. Note: the API does not verify if the device id's are valid
	setVideoInputDeviceId(deviceId: string, reconnect: boolean /* default false*/): void;
	// Sets the audio input device. Note: the API does not verify if the device id's are valid
	setAudioInputDeviceId(deviceId: string, reconnect: boolean /* default false*/): void;
	// Sets the audio output device. Note: the API does not verify if the device id's are valid
	// At the moment only the Chrome and Edge browsers support this API
	setAudioOutputDeviceId(deviceId: string, reconnect: boolean /* default false*/): void;
	// Sets the main video resolution
	setVideoResolution(resolution: MeetingRoomVideoResolution, reconnect: boolean /* default false*/): void;
	// Retrieves the currently selected media devices
	getMediaDeviceSettings(): IMediaDeviceSettings;
	// Electron applications may want to intercept screensharing requests by users
	// This allows the Electron application to pre-select a device ID
	setScreensharingInterceptor(callback: () => void): void;
	// To be used with the screensharing interceptor, throws error if no interceptor set
	startScreensharing(sourceId?: string): void;
	// To be used with the screensharing interceptor, throws error if no interceptor set
	stopScreensharing(): void;
	// works only if partially recorded is configured for this meeting (meeting.recordingType === 'partial')
	startRecording(): void;
	// works only if partially recorded is configured for this meeting (meeting.recordingType === 'partial')
	stopRecording(): void;
	// Only allowed by a moderator. The event will be blocked by the server if called by a non-moderator
	forceStopScreensharing(): void;
	// This fails in most browsers except Google Chrome: entering fullscreen requires a user interaction
	enterVideoFullscreen(): void;
	// Sends a group chat message
	sendChatMessage(message: string): void;
	// Sends a private chat message to the participant with the ID participantId
	sendPrivateChatMessage(message: string, participantId: string): void;
	// Sends a custom message to all participants
	sendCustomMessage(message: ICustomMessage): void;
	// Disconnects from the server and leaves the meeting room
	leaveMeeting(): void;
	// Allows API users to define how the videos are displayed
	// See separate documentation for more details
	setVideoDisplayCalculator(videoDisplayCalculator: IVideoDisplayCalculator): void;
	// Event listener for meeting room events
	on(event: MeetingRoomApiEvent, callback: ApiEventCallback): void;
}

export interface IMeetingConnectionConfig {
	meetingId: string;
	moderatorToken?: string;
	participantName: string;
	participantEmail?: string;
	audio: boolean;
	video: boolean;
}

export interface IVeetingBlocksApi extends IMeetingRoomApi {
	loadMeeting(meetingId: string): Promise<IMeetingRoomConfig>;
	joinMeeting(connectionConfig: IMeetingConnectionConfig): Promise<void>;
}
